#! /usr/bin/Rscript --vanilla
# -*- tab-width:2;indent-tabs-mode:t;show-trailing-whitespace:t;rm-trailing-spaces:t -*-
# vi: set ts=2 noet:
#
# (c) Copyright Rosetta Commons Member Institutions.
# (c) This file is part of the Rosetta software suite and is made available under license.
# (c) The Rosetta software is developed by the contributing members of the Rosetta Commons.
# (c) For more information, see http://www.rosettacommons.org. Questions about this can be
# (c) addressed to University of Washington UW TechTransfer, email: license@u.washington.edu.


###################################################################
#
#plot a polynomial from a hydrogen bond parameter set
#
#  cd minirosetta_database/scoring/score_functions/hbonds
#  ./scripts/plot_potential_function.R --help
#
###################################################################


# install packages: with install.packages("<PACKAGE_NAME>")

# to convert parametrizations into database
library("RSQLite")

# represent polynomials
library("polynom")

# plot functionality
library("ggplot2")

# parse command line options
library("optparse")

##### Setup command line options #########
option_list <- list(
  make_option(c("-s", "--parameter_set"), default="standard_params",
		help="Parameter set where the polynomial is defined [default: %default]"),
	make_option(c("-p", "--polynomial"), default="poly_AHdist_3",
		help="Name of the polynomial to plot [default: %default]"),
  make_option(c("-a", "--output_prefix"), default="polynomial_potential_",
		help="Prefix of filename where the plot is saved [default: %default]"),
	make_option(c("-z", "--output_suffix"), default=".png",
		help="Suffix of filename where the plot is saved [default: %default]"),
	make_option(c("-l", "--list"), action="store_true", default=FALSE,
		help="List all polynomials for specified parameter set and exit"))
opt <- parse_args(OptionParser(option_list=option_list))


##### Validate input ##########
cwd <- getwd();
if(substr(cwd, nchar(cwd)-29,nchar(cwd)) != "scoring/score_functions/hbonds"){
	stop("ERROR: Please run this script from $ROSETTA3_DB/scoring/score_functions/hbonds.")
}

if(!file.exists(opt$parameter_set)){
	stop(paste("Parameter set, ", opt$parameter_set, ", is not recognized.",sep=""))
}

###### Convert parameters into database #######

#It's necessary to actually go into the folder because looks to import
#the .csv files with in the current directory.

setwd(opt$parameter_set)
system("sqlite3 params.db3 < schema.sql")
setwd(cwd)

con <- dbConnect("SQLite", paste(opt$parameter_set, "/params.db3", sep=""))
polynomials <- dbGetQuery(con,"SELECT * FROM HBPoly1D;")


get_1d_polynomial_by_name <- function(polynomials, name){
  df <- polynomials[polynomials$name==name,]
  if (nrow(df) !=1){
    stop(paste("Polynomial, ", name," could not be found", sep=""))
  }
  #we don't know the order in which the rows columns selected will be returned!
  switch(df$degree,
         p <- df[1,c("c_a")],
         p <- df[1,c("c_b","c_a")],
         p <- df[1,c("c_c","c_b","c_a")],
         p <- df[1,c("c_d","c_c","c_b","c_a",)],
         p <- df[1,c("c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_f","c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_g","c_f","c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_h","c_g","c_f","c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_i","c_h","c_g","c_f","c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_j","c_i","c_h","c_g","c_f","c_e","c_d","c_c","c_b","c_a")],
         p <- df[1,c("c_k","c_j","c_i","c_h","c_g","c_f","c_e","c_d","c_c","c_b","c_a")])
  polynomial(p)
}

# If the user request the --list option, make the table and then exit.
if(opt$list){
	polynomials <- adply(polynomials, 1, function(polynomial){
		name <- polynomial[1, "name"]
		polynomial$equation <- as.character(signif(get_1d_polynomial_by_name(polynomials,name), digits=4), decreasing=TRUE)
		polynomial
	})
	cat("Plynomials defined in the parameter set ", opt$parameter_set,"\n")
	print(polynomials[,c("name", "equation")], right=FALSE)
	quit()
}

#########  Describe the plot that is going to be made ############
output_fname <- paste(
  opt$output_prefix, opt$parameter_set, "_", opt$polynomial, opt$output_suffix,
	sep="")

cat("Making plot of polynomial parameter:\n")
cat("    parameter_set: ", opt$parameter_set, "\n")
cat("    polynomial:    ", opt$polynomial, "\n")
cat("    output plot:   ", output_fname, "\n")


######### Make the plot ################
p <- get_1d_polynomial_by_name(polynomials, opt$polynomial)
xmin <- polynomials[polynomials$name==opt$polynomial, "xmin"]
xmax <- polynomials[polynomials$name==opt$polynomial, "xmax"]
dimension <- polynomials[polynomials$name==opt$polynomial, "dimension"]
polynomial_evaluated <- data.frame(
	  name=opt$polynomial,
	  dimension=seq(xmin, xmax, length.out=50),
	  energy=predict(p, seq(xmin, xmax, length.out=50)))

p <- ggplot(polynomial_evaluated) + theme_bw() +
  geom_hline(yintercept=0, size=.7) +
  geom_line(aes(x=dimension, y=energy), size=1.5) +
  opts(title=paste("Hydrogen Bond Polynomial Evaluation:", opt$polynomial)) +
  scale_x_continuous(dimension) +
  scale_y_continuous("Raw Energy")
ggsave(output_fname)
